// Copyright (c) 2014 GeometryFactory (France)
// All rights reserved.
//
// This file is part of CGAL (www.cgal.org).
// You can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation,
// either version 3 of the License, or (at your option) any later version.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $URL$
// $Id$
// SPDX-License-Identifier: GPL-3.0+
//
// Author(s)     : Phlipp Moeller
#ifndef CGAL_TOP_LEVEL_SURFACE_MESH_H
#define CGAL_TOP_LEVEL_SURFACE_MESH_H

#include <CGAL/license/Surface_mesh.h>


#include <CGAL/Surface_mesh/Surface_mesh_fwd.h>
#include <CGAL/Surface_mesh/Surface_mesh.h>

#ifdef DOXYGEN_RUNNING
namespace CGAL {

// fwdS for the public interface
template<typename K>
class Surface_mesh;

}
#endif

#endif /* CGAL_TOP_LEVEL_SURFACE_MESH_H */
