// Copyright (c) 2010 GeometryFactory
// All rights reserved.
//
// This file is part of CGAL (www.cgal.org); you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License as
// published by the Free Software Foundation; either version 3 of the License,
// or (at your option) any later version.
//
// Licensees holding a valid commercial license may use this file in
// accordance with the commercial license agreement provided with the software.
//
// This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
// WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// $URL$
// $Id$
// SPDX-License-Identifier: LGPL-3.0+
//
//
// Author(s)     : Sebastien Loriot, Laurent Rineau


#ifndef CGAL_QUOTIENT_FWD_H
#define CGAL_QUOTIENT_FWD_H

namespace CGAL{
  template < typename NT >
  struct Split_double;
  
  template <class NT_>
  class Quotient;
  
} //namespace CGAL

#endif //CGAL_QUOTIENT_FWD_H
